#include <tuple>
#include "frame.hpp"
#include "drawing_assets.hpp"
#include "painter.hpp"


class DemoPainter : public Painter {
public:
    void draw(PainterState *state, Frame *frame) const override {
        int W = frame->width, H = frame->height;
        // Размер рисунка возьмём меньше (7 / 8), чтобы он не касался границ экрана
        float a = 7.0f / 8 * ((W < H) ? W - 1 : H - 1) / sqrt(2);
        if (a < 1) return; // Если окно очень маленькое, то ничего не рисуем
        float angle = state->angle; // Угол поворота
        a = a / 2;

        Point center{W / 2, H / 2};

        draw_polyline(
                frame, COLOR{200, 30, 45},
                center.moved(a, a).rotated_around(center, angle),
                center.moved(a, -a).rotated_around(center, angle),
                center.moved(-a, -a).rotated_around(center, angle),
                center.moved(-a, a).rotated_around(center, angle)
        );


        // Рисуем описанную окружность
        draw_circle(frame, center, int(a * sqrt(2) + 0.5f), COLOR(100, 100, 250));

        // Рисуем пиксель, на который кликнул пользователь
        if (state->clicked_pixel.x >= 0 && state->clicked_pixel.x < W &&
            state->clicked_pixel.y >= 0 && state->clicked_pixel.y < H)
            frame->set_pixel(state->clicked_pixel.x, state->clicked_pixel.y, {34, 175, 60}); // Пиксель зелёного цвета
    }
};

static auto predefined_painters_ = std::make_tuple(
        DemoPainter()
);

Painter const *const predefined_painters[] = {
        &std::get<0>(predefined_painters_),
        nullptr
};